from django.shortcuts import render,redirect
from django.forms.models import modelform_factory
from django.views.generic import (
    CreateView,DeleteView,UpdateView,DetailView,
    ListView,View
)
from django.core.paginator import Paginator

from automatic_crud.generics import BaseCrudMixin
from automatic_crud.utils import get_object,get_form,build_template_name

class BaseList(BaseCrudMixin,ListView):

    def dispatch(self, request, *args, **kwargs):
        # login required validation
        validation_login_required,response = self.validate_login_required()
        if validation_login_required:
            return response
        
        # permission required validation
        validation_permissions,response = self.validate_permissions()
        if validation_permissions:
            return response

        return super().dispatch(request, *args, **kwargs)    

    def get_queryset(self):
        return self.model.objects.filter(model_state = True)

    def get_context_data(self, **kwargs):
        context = {}
        data = self.get_queryset()
        
        if self.model.normal_pagination:
            paginator = Paginator(data,self.model.values_for_page)
            page_number = self.request.GET('page')
            data = paginator.get_page(page_number)
        
        context['object_list'] = data
        return context

    def get(self,request,*args,**kwargs):
        self.template_name = build_template_name(self.template_name,self.model,'list')
        return render(request,self.template_name,self.get_context_data())

class BaseCreate(BaseCrudMixin,CreateView):
    
    def dispatch(self, request, *args, **kwargs):
        # login required validation
        validation_login_required,response = self.validate_login_required()
        if validation_login_required:
            return response
        
        # permission required validation
        validation_permissions,response = self.validate_permissions()
        if validation_permissions:
            return response

        return super().dispatch(request, *args, **kwargs)    

    def get(self,request,form = None,*args,**kwargs):
        self.template_name = build_template_name(self.template_name,self.model,'create')
        form = get_form(form,self.model)
        return render(request,self.template_name,{'form':form})    

    def post(self,request,form = None,*args,**kwargs):
        self.template_name = build_template_name(self.template_name,self.model,'list')
        form = get_form(form,self.model)
        
        if self.form_class == None:    
            form = form(request.POST,request.FILES)
        else:
            form = self.form_class(request.POST,request.FILES)     
        
        if form.is_valid():
            form.save()
            return redirect(self.success_url)
        else:
            form = self.form_class()
            context = {
                'form':form
            }
            return render(request,self.template_name, context)

class BaseDetail(BaseCrudMixin,DetailView):
    def dispatch(self, request, *args, **kwargs):
        # login required validation
        validation_login_required,response = self.validate_login_required()
        if validation_login_required:
            return response
        
        # permission required validation
        validation_permissions,response = self.validate_permissions()
        if validation_permissions:
            return response

        return super().dispatch(request, *args, **kwargs)    

    def get_context_data(self, **kwargs):
        context = {}
        context['object'] = get_object(self.model,self.kwargs['pk'])
        return context  

    def get(self,request,form = None,*args,**kwargs):
        self.template_name = build_template_name(self.template_name,self.model,'detail')
        return render(request,self.template_name,self.get_context_data())

class BaseUpdate(BaseCrudMixin,UpdateView):

    def dispatch(self, request, *args, **kwargs):
        # login required validation
        validation_login_required,response = self.validate_login_required()
        if validation_login_required:
            return response
        
        # permission required validation
        validation_permissions,response = self.validate_permissions()
        if validation_permissions:
            return response

        return super().dispatch(request, *args, **kwargs)   
    
    def get_context_data(self, **kwargs):
        context = {}
        context['object'] = get_object(self.model,self.kwargs['pk'])
        return context    

    def get(self,request,form = None,*args,**kwargs):
        self.template_name = build_template_name(self.template_name,self.model,'update')
        
        form = get_form(form,self.model)
        form = form(instance = get_object(self.model,self.kwargs['pk']))
        
        context = self.get_context_data()
        context['form'] = form
        return render(request,self.template_name,context)

    def post(self,request,form = None,*args,**kwargs):
        self.template_name = build_template_name(self.template_name,self.model,'list')
        form = get_form(form,self.model)
        
        instance = get_object(self.model,self.kwargs['pk'])
        if instance is not None:
            if self.form_class == None:
                form = form(request.POST,request.FILES, instance = instance)
            else:
                form = self.form_class(request.POST,request.FILES, instance = instance)   
            if form.is_valid():
                form.save()
                return redirect(self.success_url)
            else:
                form = self.form_class()
                context = {
                    'form':form
                }
                return render(request,self.template_name, context)
        else:
            return redirect(self.success_url)

class BaseDirectDelete(BaseCrudMixin,DeleteView):
    def dispatch(self, request, *args, **kwargs):
        # login required validation
        validation_login_required,response = self.validate_login_required()
        if validation_login_required:
            return response
        
        # permission required validation
        validation_permissions,response = self.validate_permissions()
        if validation_permissions:
            return response

        return super().dispatch(request, *args, **kwargs)    

class BaseLogicDelete(BaseCrudMixin,DeleteView):
    model = None

    def dispatch(self, request, *args, **kwargs):
        # login required validation
        validation_login_required,response = self.validate_login_required()
        if validation_login_required:
            return response
        
        # permission required validation
        validation_permissions,response = self.validate_permissions()
        if validation_permissions:
            return response

        return super().dispatch(request, *args, **kwargs)

    def delete(self,request,model,*args,**kwargs):
        self.model = model
        instance = get_object(self.model,self.kwargs['pk'])
        
        if instance is not None:
            instance.update(model_state = False)
            return redirect(self.success_url)        
        else:
            return redirect(self.success_url)


from datetime import datetime

from django.http import HttpResponse
from django.views.generic import TemplateView

from openpyxl import Workbook
from openpyxl.styles import (
    Alignment,Border,Font,Side
)
try:
    from openpyxl.cell import get_column_letter
except ImportError:
    from openpyxl.utils import get_column_letter

from automatic_crud.generics import BaseCrudMixin
from automatic_crud.utils import (
    get_model,get_model_fields_names,get_queryset
)

def _excel_report_title(__model_name: str):
    """
    Build report title with today date
    """


    date = datetime.now()
    title = "REPORTE DE {0} EN FORMATO EXCEL REALIZADO EN LA FECHA: {1}".format(
                                                                            __model_name.upper(),
                                                                            "%s/%s/%s" % (
                                                                                date.day,date.month,
                                                                                date.year
                                                                            )
                                                                        )
    return title

def _validate_id(__field: str):
    if str(__field).lower() != 'id':
        return True
    return False

class ExcelReportFormat:
    """
    This class generates a report in excel for any model you want, 
    the parameters to be used are defined in the constructor, 
    and there are also some methods that build the report block by block.

    Parameters:
        _app_name                   name of the application where is the model to be used.
        _model_name                 name of the model to be used.

    Variables:
        _app_name                   name of the application where is the model to be used.
        _model_name                 name of the model to be used.
        __model                     model to be used
        __model_fields_names        fields list of model.
        __queryset                  queryset of model, contains all registers of model.
        __report_title              report title.
        __workbook                  Workbook instance, Excel workbook.
        __sheetwork                 Excel Sheetwork, by default first sheet.

    """
    

    def __init__(self,__app_name:str,__model_name:str, *args, **kwargs):
        self.__app_name = __app_name
        self.__model_name = __model_name
        self.__model = get_model(self.__app_name,self.__model_name)
        self.__model_fields_names = get_model_fields_names(self.__model)
        self.__queryset = get_queryset(self.__model)
        self.__report_title = _excel_report_title(self.__model_name)
        self.__workbook = Workbook()
        self.__sheetwork = self.__workbook.active

    def get_model(self):
        return self.__model

    def __excel_report_header(self,row_dimension = 15, col_dimension = 25):
        """
        Build excel report header, print report title and add default styles

        """

        self.__sheetwork['B1'].alignment = Alignment(horizontal = "center", vertical = "center")
        self.__sheetwork['B1'].border = Border(left = Side(border_style = "thin"), right = Side(border_style = "thin"),
                                                top = Side(border_style = "thin"), bottom = Side(border_style = "thin"))
        self.__sheetwork['B1'].font = Font(name = 'Calibri', size = 12, bold = True)
        self.__sheetwork['B1'] = self.__report_title
        
        if len(self.__model_fields_names) < 12:
            __header_letter = 'L'
        else:
            __header_letter = '{0}'.format(get_column_letter(len(self.__model_fields_names)).upper())
        
        self.__sheetwork.merge_cells('B1:{0}1'.format(__header_letter))
        self.__sheetwork.row_dimensions[3].height = row_dimension

        __count = 1
        for __field in self.__model_fields_names:
            __letter = get_column_letter(__count).upper()
            self.__sheetwork['{0}3'.format(__letter)].alignment = Alignment(horizontal = "center", vertical = "center")
            self.__sheetwork['{0}3'.format(__letter)].border = Border(left = Side(border_style = "thin"), right = Side(border_style = "thin"),
                                                top = Side(border_style = "thin"), bottom = Side(border_style = "thin"))
            self.__sheetwork['{0}3'.format(__letter)].font = Font(name = 'Calibri', size = 9, bold = True)
            self.__sheetwork['{0}3'.format(__letter)] = '{0}'.format(__field.upper())
            self.__sheetwork.column_dimensions['{0}'.format(__letter)].height = col_dimension
            __count += 1
    
    def __print_values(self):
        """
        Print values of queryset, skip id value and exclude fields of model
        """

        row_count = 4
        col_count = 1
        general_count = len(self.__model_fields_names)
        data = [value for value in self.__queryset]

        for value in data:
            for key,subvalue in value.items():
                if _validate_id(key):
                    if key not in self.__model.exclude_fields:
                        if general_count > 0:
                            self.__sheetwork.cell(row = row_count, column = col_count).alignment = Alignment(horizontal = "center")
                            self.__sheetwork.cell(row = row_count, column = col_count).border = Border(left = Side(border_style = "thin"),
                                                                        right = Side(border_style = "thin"),top = Side(border_style = "thin"), 
                                                                        bottom = Side(border_style = "thin"))
                            if type(subvalue) is bool:
                                if subvalue is True:
                                    self.__sheetwork.cell(row = row_count, column = col_count).value = 'No eliminado'
                                else:
                                    self.__sheetwork.cell(row = row_count, column = col_count).value = 'Eliminado'
                            else:
                                self.__sheetwork.cell(row = row_count, column = col_count).value = str(subvalue)
                                if (self.__sheetwork.column_dimensions[get_column_letter(col_count).upper()].width < len(str(subvalue))):
                                    self.__sheetwork.column_dimensions[get_column_letter(col_count).upper()].width = len(str(subvalue))
                            
                            col_count += 1
                            general_count -= 1

                    if general_count == 0:
                        general_count = len(self.__model_fields_names)
            
            row_count += 1
            col_count = 1

    def get_excel_report(self):
        """
        Generate excel response using model name
        """

        report_name = "Reporte {0} en Excel .xlsx".format(self.__model_name)
        response = HttpResponse(content_type = "application/ms-excel")
        content = "attachment; filename = {0}".format(report_name)
        response['Content-Disposition'] = content
        self.__workbook.save(response)
        return response

    def build_report(self):
        """
        Build report call 2 functions: __excel_report_header and __print_values
        """

        self.__excel_report_header()
        self.__print_values()

class GetExcelReport(BaseCrudMixin,TemplateView):
    """
    Return Instance Excel Report for a model.
    """

    def get(self,request,_app_name:str,_model_name:str,*args,**kwargs):
        __report = ExcelReportFormat(_app_name,_model_name)

        self.model = __report.get_model()

        # login required validation
        validation_login_required,response = self.validate_login_required()
        if validation_login_required:
            return response
        
        # permission required validation
        validation_permissions,response = self.validate_permissions()
        if validation_permissions:
            return response

        __report.build_report()
        return __report.get_excel_report()


import os
from setuptools import setup

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()


setup(
    name='django-automatic-crud',
    version='1.0.7',
    packages=['automatic_crud'],
    include_package_data=True,
    license='BSD License',
    description='CRUDS Automáticos con Django',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/developerpe/django-automatic-crud',
    author='Oliver Sandoval',
    author_email='developerpeperu@gmail.com',
    install_requires=[
        'Django>=2.2',
        'openpyxl==3.0.7',
    ],
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.3'
    ]
)